@@ -48,7 +48,12 @@ from google.cloud.sqlalchemy_spanner._opentelemetry_tracing import trace_call
 @listens_for(Pool, "reset")
 def reset_connection(dbapi_conn, connection_record):
     """An event of returning a connection back to a pool."""
-    dbapi_conn.connection.staleness = None
+    dbapi_conn.connection.rollback()
+    if getattr(dbapi_conn.connection, "staleness", None) is not None:
+        dbapi_conn.connection.staleness = None
+
+    if getattr(dbapi_conn.connection, "read_only", None) is not None:
+        dbapi_conn.connection.read_only = False
 
 
 # register a method to get a single value of a JSON object
@@ -324,6 +329,29 @@ class SpannerSQLCompiler(SQLCompiler):
 class SpannerDDLCompiler(DDLCompiler):
     """Spanner DDL statements compiler."""
 
+    def get_column_specification(self, column, **kwargs):
+        """Build new column specifications.
+
+        Overridden to move the NOT NULL statement to front
+        of a computed column expression definitions.
+        """
+        colspec = (
+            self.preparer.format_column(column)
+            + " "
+            + self.dialect.type_compiler.process(column.type, type_expression=column)
+        )
+        default = self.get_column_default_string(column)
+        if default is not None:
+            colspec += " DEFAULT " + default
+
+        if not column.nullable:
+            colspec += " NOT NULL"
+
+        if column.computed is not None:
+            colspec += " " + self.process(column.computed)
+
+        return colspec
+
     def visit_computed_column(self, generated, **kw):
         """Computed column operator."""
         text = "AS (%s) STORED" % self.sql_compiler.process(
@@ -471,7 +499,7 @@ class SpannerDialect(DefaultDialect):
     Represents an API layer to control Cloud Spanner database with SQLAlchemy API.
     """
 
-    name = "spanner"
+    name = "spanner+spanner"
     driver = "spanner"
     positional = False
     paramstyle = "format"
@@ -488,6 +516,7 @@ class SpannerDialect(DefaultDialect):
     supports_native_enum = True
     supports_native_boolean = True
     supports_native_decimal = True
+    supports_statement_cache = True
 
     ddl_compiler = SpannerDDLCompiler
     preparer = SpannerIdentifierPreparer
